from django.conf import settings
import os
import defaults

def patch_settings():
	if not hasattr(settings, 'PROJECT_ROOT'):
		settings.PROJECT_ROOT = defaults.PROJECT_ROOT
	
	if not hasattr(settings, 'PROJECT_PARENT_DIR'):
		settings.PROJECT_PARENT_DIR = os.path.dirname(settings.PROJECT_ROOT)
	
	if not hasattr(settings, 'LOG_DIR'):
		settings.LOG_DIR = defaults.LOG_DIR
	
	if not hasattr(settings, 'CONFIG_GEN_TEMPLATES_DIR'):
		settings.CONFIG_GEN_TEMPLATES_DIR = defaults.TEMPLATES_DIR
	
	if not hasattr(settings, 'CONFIG_GEN_GENERATED_DIR'):
		settings.CONFIG_GEN_GENERATED_DIR = defaults.GENERATED_DIR
	
	if not hasattr(settings, 'HOST'):
		settings.HOST = defaults.HOST
	
	if settings.ADMIN_MEDIA_PREFIX[:len(settings.MEDIA_URL)] == settings.MEDIA_URL:
		settings.ADMIN_MEDIA_IN_MEDIA = True



import __main__
import os
from django.contrib.sites.models import Site
from django.db import transaction

PROJECT_ROOT = os.path.abspath(os.path.dirname(__main__.__file__))
LOG_DIR = os.path.join(PROJECT_ROOT ,'logs')

_config_dir = os.path.join(PROJECT_ROOT ,'config')
TEMPLATES_DIR = os.path.join(_config_dir ,'templates')
GENERATED_DIR = os.path.join(_config_dir ,'generated')

#Error is thrown when running "./manage.py syncdb" for the first time and this app is installed.
try:
	HOST = Site.objects.get_current().domain.split(':')[0]
except:
	transaction.rollback()
	HOST = 'localhost'




from django.core.management.base import NoArgsCommand, CommandError
from django.conf import settings
from .. import patch_settings
import json
import copy
import logging

logger = logging.getLogger(__name__)

class NullHandler(logging.Handler):
	def emit(self, record):
		pass

patch_settings()

class Command(NoArgsCommand):
	help = 'Prints out settings serialized as JSON.'
	
	def handle_noargs(self, **options):
		#remove logging statements from output
		l = logging.getLogger('')
		for h in l.handlers:
			l.removeHandler(h)
		l.addHandler(NullHandler())
		
		d = {}
		s_d = settings._wrapped.__dict__
		for key in settings._wrapped.__dict__:
			val = s_d[key]
			logger.debug('%s: %s' % (key, val))
			try:
				#if settings has something like "import django.conf.global_settings as DEFAULT_SETTINGS"
				#in it, then json encoding will throw and error. Copying makes
				#sure modules don't get included.
				d[key] = copy.copy(val)
			except Exception as e:
				logger.error(e)
		print json.dumps(d, indent=4, sort_keys=True)



from django.core.management.base import NoArgsCommand, CommandError
from django.conf import settings
from django.template.loader import render_to_string
from django.template import Template, Context
from .. import patch_settings
import os
import shutil
import logging

logger = logging.getLogger(__name__)

patch_settings()

TEMPLATES_DIR = settings.CONFIG_GEN_TEMPLATES_DIR
#logger.debug(TEMPLATES_DIR)
GENERATED_DIR = settings.CONFIG_GEN_GENERATED_DIR
#logger.debug(GENERATED_DIR)

class Command(NoArgsCommand):
	help = 'Generates configuration files for Apache, Nginx, etc. using values in settings.py and the Django template system.'
	ctx = None
	
	def handle_noargs(self, **options):
		#get all templates in TEMPLATES_DIR, parse them, and output files in GENERATED_DIR
		#logging.debug(settings)
		self.ctx = Context(settings._wrapped.__dict__)
		
		if not os.path.exists(TEMPLATES_DIR):
			os.makedirs(TEMPLATES_DIR)
		
		if not os.path.exists(GENERATED_DIR):
			os.makedirs(GENERATED_DIR)
		
		dir_list=os.listdir(TEMPLATES_DIR)
		#if no templates are present, populate template directory with the examples
		if len(dir_list) is 0:
			logger.debug('%s was empty' % TEMPLATES_DIR)
			example_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'example_templates')
			for filename in os.listdir(example_dir):
				shutil.copy2(os.path.join(example_dir, filename), TEMPLATES_DIR)
			dir_list=os.listdir(TEMPLATES_DIR)
		
		self.create_nodes('', dir_list)
	
	def create_nodes(self, rel_path, dir_list):
		for filename in dir_list:
			logger.debug('Filename: %s' % filename)
			t_filename = os.path.join(TEMPLATES_DIR, rel_path, filename)
			g_filename = os.path.join(GENERATED_DIR, rel_path, filename)
			if os.path.isfile(t_filename):
				fi = open(t_filename, 'r')
				t = Template(fi.read())
				fi.close()
			
				fo = open(g_filename, 'w')
				fo.write(t.render(self.ctx))
				fo.close()
			elif os.path.isdir(t_filename):
				if not os.path.exists(g_filename):
					logger.debug('Creating directory')
					os.mkdir(g_filename)
				self.create_nodes(filename, os.listdir(t_filename))
